package cn.dankal.test.first;

import cn.dankal.test.common.Const;
import cn.hutool.core.collection.CollectionUtil;
import org.apache.kafka.clients.consumer.ConsumerConfig;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.apache.kafka.clients.consumer.ConsumerRecords;
import org.apache.kafka.clients.consumer.KafkaConsumer;
import org.apache.kafka.common.serialization.StringDeserializer;
import org.junit.jupiter.api.AfterAll;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;

import java.time.Duration;
import java.time.LocalDateTime;
import java.util.Objects;
import java.util.Properties;

/**
 * 测试 Kafka clients: 第一个 consumer
 *
 * @author ZGH.MercyModest
 * @version V1.0.0
 * @create 2021-01-25
 */
public class KafkaFirstConsumer {


    /**
     * KafkaConsumer
     */
    private static KafkaConsumer<String, String> kafkaConsumer;


    @BeforeAll
    public static void before() {
        Properties properties = new Properties();
        final String bootstrapServers = "192.168.128.3:9092,192.168.128.3:9093,192.168.128.3:9094";
        // broker(Kafka服务器) 地址清单
        properties.put(ConsumerConfig.BOOTSTRAP_SERVERS_CONFIG, bootstrapServers);
        // message key 反序列化器
        properties.put(ConsumerConfig.KEY_DESERIALIZER_CLASS_CONFIG, StringDeserializer.class.getCanonicalName());
        // message value 反序列化器
        properties.put(ConsumerConfig.VALUE_DESERIALIZER_CLASS_CONFIG, StringDeserializer.class.getCanonicalName());

        properties.put(ConsumerConfig.GROUP_ID_CONFIG, Const.Kafka.ConsumerGroupId.HELLO_CONSUMER_GROUP_ID);

        // 通过 Properties 配置 实例化 KafkaConsumer
        kafkaConsumer = new KafkaConsumer<>(properties);
    }

    /**
     * 第一个 Kafka 消费者
     */
    @Test
    public void testKafkaFirstConsumer() {
        // consumer 订阅 主题
        kafkaConsumer.subscribe(CollectionUtil.newArrayList(Const.Kafka.Topic.HELLO_TOPIC));
        while (true) {
            ConsumerRecords<String, String> consumerRecords = kafkaConsumer.poll(Duration.ofMillis(500));
            for (ConsumerRecord<String, String> consumerRecord : consumerRecords) {
                System.out.printf(
                        "%s 消息消息 ==> 主题: %s 分区: %s 偏移量: %s key: %s value: %s %n",
                        Const.FULL_DATE_TIME_FORMATTER.format(LocalDateTime.now()),
                        consumerRecord.topic(),
                        consumerRecord.partition(),
                        consumerRecord.offset(),
                        consumerRecord.key(),
                        consumerRecord.value());
            }
        }
    }

    @AfterAll
    public static void after() {
        if (Objects.nonNull(kafkaConsumer)) {
            kafkaConsumer.close();
        }
    }
}
